-- ============================================================================
-- == HalonaTools - A collection of functions used to help users monitor     ==
-- ==               their buffs.                                             ==
-- ============================================================================

HTBuffMonitor = LibStub("AceAddon-3.0"):NewAddon("HTBuffMonitor", "AceConsole-3.0", "AceEvent-3.0")
local L       = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

-- Frame required to permit tooltips to be scanned for temp-buffs like poisons.
local TooltipScanner = CreateFrame('GameTooltip', 'TooltipScanner', WorldFrame, 'GameTooltipTemplate')

-- Some checks, triggered by events, should only happen once until reset event.
local isPrimedCombatCheck = true

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTBuffMonitor:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTBuffMonitor:OnEnable()

  -- Subscribe to events that are used to trigger buff checks.
  self:RegisterEvent("READY_CHECK")
  self:RegisterEvent("PLAYER_REGEN_ENABLED")
  self:RegisterEvent("PLAYER_REGEN_DISABLED")
  self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

  -- Declare the tooltip scanner's owner. Used to check weapon buffs.
  TooltipScanner:SetOwner(WorldFrame, "ANCHOR_NONE")

end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTBuffMonitor:OnDisable()
end

-- ============================================================================
-- == DoMissingBuffCheck(characterName, channelName)                         ==
-- ==                                                                        ==
-- == This function will check the named character for missing buffs.        ==
-- ============================================================================

function HTBuffMonitor:DoMissingBuffCheck(characterName, channelName)

  local hasSuitableFlask          = HTBuffMonitor:IsBuffedWithSuitableFlask(characterName)
  local hasSuitableBattleElixir   = HTBuffMonitor:IsBuffedWithSuitableBattleElixir(characterName)
  local hasSuitableGuardianElixir = HTBuffMonitor:IsBuffedWithSuitableGuardianElixir(characterName)
  local hasSuitableFoodBuff       = HTBuffMonitor:IsBuffedWithSuitableFood(characterName)
  local hasSuitablePoisonBuffs    = HTBuffMonitor:IsBuffedWithSuitableRoguePoisons(characterName)

  -- == -----------------------------------------------------------------------
  -- == -- Check if character has a flask or two elixirs activated.          --
  -- == -----------------------------------------------------------------------
  if not hasSuitableFlask then

    -- == ---------------------------------------------------------------------
    -- == -- Has guardian elixir but suitable battle elixir missing.         --
    -- == ---------------------------------------------------------------------
    if  not hasSuitableBattleElixir
    and     hasSuitableGuardianElixir then
      if HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-battle-elixir"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end

    -- == ---------------------------------------------------------------------
    -- == -- Has battle elixir but suitable guardian elixir missing.         --
    -- == ---------------------------------------------------------------------
    if      hasSuitableBattleElixir
    and not hasSuitableGuardianElixir then
      if HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-guardian-elixir"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end

    -- == ---------------------------------------------------------------------
    -- == -- No flask, no battle elixir, no guardian elixir.                 --
    -- == ---------------------------------------------------------------------
    if  not hasSuitableBattleElixir
    and not hasSuitableGuardianElixir then
      if HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-no-flask-or-elixirs"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end

  end

  -- == -----------------------------------------------------------------------
  -- == -- Check if character has a food-buff activated.                     --
  -- == -----------------------------------------------------------------------
  if not hasSuitableFoodBuff then
    if HTConfig:GetIsEnabledBuffMonitorCheckFood(nil) then
      HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-food"], "CHARACTER_NAME", characterName), channelName, nil, nil)
    end
  end

  -- == -----------------------------------------------------------------------
  -- == -- Check if character has suitable rogue poisons activated.          --
  -- == -----------------------------------------------------------------------
  if UnitClass(characterName) == L["all-data-classname-rogue"] then
    if not hasSuitablePoisonBuffs then
      if HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-weapon-poisons"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end
  end

  -- == -----------------------------------------------------------------------
  -- == -- Check if character buffs are nearing expiry.                      --
  -- == -----------------------------------------------------------------------
  if HTConfig:GetIsEnabledBuffMonitorExpiryTimeCheck(nil) then
     HTBuffMonitor:CheckExpiryTimes(characterName, channelName)
  end

end

-- ============================================================================
-- == IsCharacterBuffedWithSuitableFlask(characterName)                      ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable flask activated.                                              ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableFlask(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableFlasksArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableBattleElixir(characterName)                        ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable battle elixir activated.                                      ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableBattleElixir(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableBattleElixirsArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableGuardianElixir(characterName)                      ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable guardian elixir activated.                                    ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableGuardianElixir(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableGuardianElixirsArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableFood(characterName)                                ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable food buff activated.                                          ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableFood(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableFoodArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableRoguePoisons(characterName)                        ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has       ==
-- == suitable rogue poisons on their weapons.                               ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableRoguePoisons(characterName)

  -- == ----------------------------------------------------------------------
  -- == -- If it is impossible to inspect the player, return the default    --
  -- == -- response. This is currently 'true' because we do not want an     --
  -- == -- inability to inspect to trigger 'false-positives'.               --
  -- == ----------------------------------------------------------------------
  if not CanInspect(characterName) then
    return true
  end

  -- == ----------------------------------------------------------------------
  -- == -- Get the array of 'suitable' rogue poisons.                       --
  -- == ----------------------------------------------------------------------
  local suitableArray = HTBuffMonitor:GetSuitableRoguePoisonsArray()

  -- == ----------------------------------------------------------------------
  -- == -- Initialise a counter for rogue poison buffs.                     --
  -- == ----------------------------------------------------------------------
  local poisonBuffCount = 0

  -- == ----------------------------------------------------------------------
  -- == -- Advise server of imminent character inspection.                  --
  -- == ----------------------------------------------------------------------
  NotifyInspect(characterName)

  -- == ----------------------------------------------------------------------
  -- == -- Check the character's mainhand weapon.                           --
  -- == ----------------------------------------------------------------------
  TooltipScanner:ClearLines()
  TooltipScanner:SetInventoryItem(characterName, 16)

  if TooltipScanner:NumLines() < 1 then
    -- HTConsole:Debug("tooltipscanner numlines is <1")
  else
    for i,v in ipairs(suitableArray) do
      if HTBuffMonitor:FindInTooltips(v) then
        poisonBuffCount = poisonBuffCount + 1
      end
    end
  end

  -- == ----------------------------------------------------------------------
  -- == -- Check the character's offhand weapon.                           --
  -- == ----------------------------------------------------------------------
  TooltipScanner:ClearLines()
  TooltipScanner:SetInventoryItem(characterName, 17)

  if TooltipScanner:NumLines() < 1 then
    -- HTConsole:Debug("tooltipscanner numlines is <1")
  else
    for i,v in ipairs(suitableArray) do
      if HTBuffMonitor:FindInTooltips(v) then
        poisonBuffCount = poisonBuffCount + 1
      end
    end
  end

  if poisonBuffCount == 2 then
    return true
  else
    return false
  end

end

-- ============================================================================
-- == FindInTooltips(textToFind)                                             ==
-- ==                                                                        ==
-- == This function simply examines an item's tooltip looking for the text   ==
-- == string supplied.                                                       ==
-- ============================================================================

function HTBuffMonitor:FindInTooltips(textToFind)

  textToFind = HTData:SafeString(textToFind)

  local numTooltipLines = TooltipScanner:NumLines()
  for tooltipLineNumber = 1, numTooltipLines do
    local tooltipTextLine = getglobal('TooltipScannerTextLeft' .. tooltipLineNumber):GetText()
    -- HTConsole:Debug("FindInTooltips(" .. HTData:SafeString(textToFind) .. ") discovered: [" .. HTData:SafeString(tooltipTextLine) .. "]")
    if tooltipTextLine then
      if string.find(tooltipTextLine, textToFind) then
        return true
      end
    else
      return false
    end
  end

  return false

end

-- ============================================================================
-- == IsBuffedWith(characterName, suitableBuffs)                             ==
-- ==                                                                        ==
-- == This function performs a check on the character identified by the      ==
-- == supplied parameter to determine if that character is currently buffed  ==
-- == with any of the buffs detailed in the supplied 'suitableBuffs' table.  ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWith(characterName, suitableBuffs)

  -- == ----------------------------------------------------------------------
  -- == -- Perform the test.                                                --
  -- == ----------------------------------------------------------------------
  for b = 1, 64 do
    local buffName = UnitBuff(characterName, b)
    if buffName then
      for i,v in ipairs(suitableBuffs) do
        if suitableBuffs[i] == buffName then
          return true
        end
      end
    end
  end

  return false

end

-- ============================================================================
-- == CheckExpiryTimes(characterName, channelName)                           ==
-- ==                                                                        ==
-- == This function simply checks the expiry times of the buff-types that    ==
-- == HalonaTools monitors and reports any buffs that will expire within a   ==
-- == prescribed time-limit to the nominated channel.                        ==
-- ============================================================================

function HTBuffMonitor:CheckExpiryTimes(characterName, channelName)

  -- HTConsole:Debug("HTBuffMonitor:CheckExpiryTimes("..characterName..", "..channelName..")")

  -- == ----------------------------------------------------------------------
  -- == -- Setup the 'suitable' buff array and other required properties.   --
  -- == ----------------------------------------------------------------------
  local suitableFlasksArray          = HTBuffMonitor:GetSuitableFlasksArray()
  local suitableBattleElixirsArray   = HTBuffMonitor:GetSuitableBattleElixirsArray()
  local suitableGuardianElixirsArray = HTBuffMonitor:GetSuitableGuardianElixirsArray()
  local suitableFoodArray            = HTBuffMonitor:GetSuitableFoodArray()
  local suitableRoguePoisonsArray    = HTBuffMonitor:GetSuitableRoguePoisonsArray()

  local timeLimitMinutes             = HTConfig:GetBuffMonitorExpiryTimeLimit(nil)

  local suitableBuffsArray           = {}

  if HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableFlasksArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableBattleElixirsArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableGuardianElixirsArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckFood(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableFoodArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableRoguePoisonsArray)
  end

  -- == ----------------------------------------------------------------------
  -- == -- Perform the tests.                                               --
  -- == ----------------------------------------------------------------------
  for b = 1, 64 do
    local buffName, buffRank, buffIcon, buffCount, buffDispelType, buffDuration, buffExpires, buffCaster, buffIsStealable = UnitBuff(characterName, b)
    if buffName then
      if buffDuration then
        for i,v in ipairs(suitableBuffsArray) do
          if suitableBuffsArray[i] == buffName then

            remainingTime = HTData:SafeNumber(tonumber(format("%.2f",-1*(GetTime()-buffExpires)/60)))

            -- HTConsole:Debug("Buff: "..buffName)
            -- HTConsole:Debug("Remaining Time: "..remainingTime..", timeLimitMinutes: "..HTData:SafeNumber(timeLimitMinutes))

            if remainingTime < timeLimitMinutes then
              local messageText = string.gsub(L["bfm-short-buff-duration-announce"], "CHARACTER_NAME", characterName)
              messageText       = string.gsub(messageText, "BUFF_NAME", buffName)
              messageText       = string.gsub(messageText, "TIME_LIMIT", timeLimitMinutes)

              HTConsole:SendChatMessage(messageText, channelName, nil, nil)

            end
          end
        end
      end
    end
  end

end

-- ============================================================================
-- == GetSuitableFlasksArray()                                               ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == flasks with respect to PVE raid usage.                                 ==
-- ============================================================================

function HTBuffMonitor:GetSuitableFlasksArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataFlasks(nil) then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79469))  -- Flask of Steelskin.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79470))  -- Flask of the Draconic Mind.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79471))  -- Flask of the Winds.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79472))  -- Flask of Titanic Strength.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(94160))  -- Flask of Flowing Water.
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathFlasks(nil) then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53760))  -- Flask of Endless Rage.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53755))  -- Flask of the Frost Wyrm.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(54212))  -- Flask of Pure Mojo.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53758))  -- Flask of Stoneblood.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(62380))  -- Lesser Flask of Resistance.
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableBattleElixirsArray()                                        ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == battle elixirs with respect to PVE raid usage.                         ==
-- ============================================================================

function HTBuffMonitor:GetSuitableBattleElixirsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataBattleElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79477)) -- Elixir of the Cobra.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79481)) -- Elixir of Impossible Accuracy.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79632)) -- Elixir of Mighty Speed.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79635)) -- Elixir of the Master.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79468)) -- Ghost Elixir.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79474)) -- Elixir of the Naga.
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathBattleElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60340)) -- Accuracy.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60345)) -- Armor Piercing.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60341)) -- Deadly Strikes.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60344)) -- Expertise.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53749)) -- Guru's Elixir.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60346)) -- Lightning Speed.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(28497)) -- Mighty Agility.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53748)) -- Mighty Strength.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(33721)) -- Spellpower Elixir.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53746)) -- Wrath Elixir.
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableGuardianElixirsArray()                                      ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == guardian elixirs with respect to PVE raid usage.                       ==
-- ============================================================================

function HTBuffMonitor:GetSuitableGuardianElixirsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataGuardianElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79480)) -- Elixir of Deep Earth.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79631)) -- Prismatic Elixir.
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathGuardianElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60340)) -- Accuracy.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53763)) -- Wrath - Elixir of Protection.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53747)) -- Wrath - Elixir of Spirit.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60343)) -- Wrath - Mighty Defense.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53751)) -- Wrath - Mighty Fortitude.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53764)) -- Wrath - Mighty Mageblood.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60347)) -- Wrath - Mighty Thoughts.
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableFoodArray()                                                 ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == food related buffs with respect to PVE raid usage.                     ==
-- ============================================================================

function HTBuffMonitor:GetSuitableFoodArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataFood() then

  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathFood() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(35272)) -- Well Fed
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableRoguePoisonsArray()                                         ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == rogue poisons for end-game content.                                    ==
-- ============================================================================

function HTBuffMonitor:GetSuitableRoguePoisonsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataPoisons() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(2892))  -- Classic -  Deadly Poison
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(6947))  -- Classic -  Instant Poison
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathPoisons() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(3408))  -- Classic -  Crippling Poison
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(5760))  -- Classic -  Mind Numbing Poison

    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57981)) -- Wrath   -  Anaesthetic Poison II
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57970)) -- Wrath   -  Deadly Poison IX
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57965)) -- Wrath   -  Instant Poison IX
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57975)) -- Wrath   -  Wound Poison VII
  end

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  --  The following matches: Anesthetic Poison, Deadly Poison [IVX]*,
  --                         Crippling Poison [IVX]*, Wound Poison [IVX]*,
  --                         Instant Poison [IVX]*, Mind-numbing Poison [IVX]*
  table.insert(suitableBuffs, "( Poison ?[IVX]*)")

  return suitableBuffs

end

-- ============================================================================
-- == GetSpellNameFromSpellInfo(spellId)                                     ==
-- ==                                                                        ==
-- == This function simply returns the name of a spell when supplied with    ==
-- == a spellId by using the standard API function GetSpellInfo(spellId) and ==
-- == retrieving the name from the data returned.                            ==
-- ============================================================================

function HTBuffMonitor:GetSpellNameFromSpellInfo(spellId)

   spellName, spellRank, spellIcon, spellPowerCost, spellIsFunnel, spellPowerType, spellCastingTime, spellMinRange, spellMaxRange = GetSpellInfo(spellId)

   spellName = HTData:SafeString(spellName)

   return spellName

end

-- ============================================================================
-- == READY_CHECK(sender)                                                    ==
-- ==                                                                        ==
-- == This function is triggered when a ready check is fired.                ==
-- ============================================================================

function HTBuffMonitor:READY_CHECK(sender)

  if HTConfig:GetIsEnabledBuffMonitor(nil) and HTConfig:GetIsEnabledBuffMonitorCheckOnReadyCheck(nil) then
    HTConsole:SendChatMessage(L["bfm-readycheck-msg-01"], "CHAT", nil, nil)
    HTBuffMonitor:DoSelfCheck()
    HTConsole:SendChatMessage(L["bfm-readycheck-msg-02"], "CHAT", nil, nil)
    HTBuffMonitor:DoRaidCheck("RAID")
  end

end

-- ============================================================================
-- == PLAYER_REGEN_DISABLED()                                                ==
-- ==                                                                        ==
-- == This function is triggered when the player enters combat.              ==
-- ============================================================================

function HTBuffMonitor:PLAYER_REGEN_DISABLED()

  HTConfig:SetIsPlayerInCombat(true, true)

  if HTConfig:GetIsEnabledBuffMonitor(nil) and HTConfig:GetIsEnabledBuffMonitorCheckOnCombatStart(nil) then
    if isPrimedCombatCheck then
      HTBuffMonitor:DoRaidCheck("CHAT")
      isPrimedCombatCheck = false
    end
  end

end

-- ============================================================================
-- == PLAYER_REGEN_ENABLED()                                                 ==
-- ==                                                                        ==
-- == This function is triggered when the player leaves combat.              ==
-- ============================================================================

function HTBuffMonitor:PLAYER_REGEN_ENABLED()

  HTConfig:SetIsPlayerInCombat(false, false)

  -- Ensures tasks may be performed when combat begins without repeating.
  isPrimedCombatCheck = true

end

-- ============================================================================
-- == COMBAT_LOG_EVENT_UNFILTERED(...)                                       ==
-- ==                                                                        ==
-- == This function parses the combat log events when they fire,             ==
-- == to determine whether they represent an event occurring that the addon  ==
-- == is expected to respond to.                                             ==
-- ============================================================================

function HTBuffMonitor:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, subevent, hideCaster, srcGUID, srcname, srcflags, dstGUID, dstname, dstflags, spellID, spellname, spellschool, extraspellID, extraspellname, extraspellschool, auratype)

  -- Attempt to capture some useful information using debug mode.
  if HTConfig:GetIsEnabledDebugMode(nil) == true then
    if HTConfig:GetIsPlayerInCombat(nil) == false then
      local characterName = string.lower(HTData:SafeString(srcname))

      -- spellID, spellname, spellschool, extraspellID, extraspellname, extraspellschool, auratype
      if characterName == "broliant" or characterName == "kenjymin" or characterName == "vanhelgen" or characterName == "koyaanis" then
        HTConsole:Debug(HTConsole:GetTitleBar());
        HTConsole:Debug("Character: [" ..        HTData:SafeString(srcname)          .. "]")
        HTConsole:Debug("subevent: [" ..         HTData:SafeString(subevent)         .. "]")
        HTConsole:Debug("spellID: [" ..          HTData:SafeString(spellID)          .. "]")
        HTConsole:Debug("spellname: [" ..        HTData:SafeString(spellname)        .. "]")
        HTConsole:Debug("spellschool: [" ..      HTData:SafeString(spellschool)      .. "]")
        HTConsole:Debug("extraspellID: [" ..     HTData:SafeString(extraspellID)     .. "]")
        HTConsole:Debug("extraspellname: [" ..   HTData:SafeString(extraspellname)   .. "]")
        HTConsole:Debug("extraspellschool: [" .. HTData:SafeString(extraspellschool) .. "]")
        HTConsole:Debug("auratype: [" ..         HTData:SafeString(auratype)         .. "]")
        HTConsole:Debug(HTConsole:GetTitleBar());
      end
    end
  end

  -- Perform the following ONLY when the player is NOT in combat.
  if HTConfig:GetIsPlayerInCombat(nil) == false then

    -- Ensure required parameters have been returned, otherwise exit immediately.
    -- *NB: The COMBAT_LOG_EVENT_UNFILTERED function signature may vary from
    -- time to time - therefore, the 'spellschool' parameter currently holds the
    -- value we consider to be the 'spellID'.
    if not subevent or not spellschool or not srcname then
      return
    end

    -- Perform the following ONLY when the player IS in a raid.
    if HTCommon:IsPlayerInRaid() then
      HTBuffMonitor:BuffAnnouncerCombatLogChecker(spellschool, subevent, srcname)
    end

  end

end

-- ============================================================================
-- == BuffAnnouncerCombatLogChecker(spellID, subevent, playerName)           ==
-- ==                                                                        ==
-- == This function determines whether the data supplied by the combat log   ==
-- == event matches a watched-for buff announcement and calls the            ==
-- == appropriate announcement function if it does.                          ==
-- ============================================================================

function HTBuffMonitor:BuffAnnouncerCombatLogChecker(spellID, subevent, playerName)

  -- Return immediately if the Buff Announcer is not enabled.
  if HTConfig:GetIsEnabledBuffAnnouncer(nil) == false then return end

  -- "Feasts" - (Fish Feast:57426), (Broiled Dragon Feast:87643),
  --            (Goblin Barbecue Feast:87915), (Seafood Magnifique Feast: 87644)
  if HTConfig:GetIsEnabledBuffAnnouncerFood(nil) == true then
    if subevent == L["bfm-log-subevent-spell-cast-start"] then
      if tonumber(spellID) == 16936 or tonumber(spellID) == 57426 or tonumber(spellID) == 87643 or tonumber(spellID) == 87915 or tonumber(spellID) == 87644 then
        HTBuffMonitor:DoBuffAnnouncerFoodAnnouncement(playerName)
        return
      end
    end
  end

  -- "Cauldrons" - (Cauldron of Battle:92649), (Big Cauldron of Battle:92712)
  if HTConfig:GetIsEnabledBuffAnnouncerCauldron(nil) == true then
    if subevent == L["bfm-log-subevent-spell-cast-start"] then
      if tonumber(spellID) == 92649 or tonumber(spellID) == 92712 then
        HTBuffMonitor:DoBuffAnnouncerCauldronAnnouncement(playerName)
        return
      end
    end
  end

  -- "Soulwells" - (Ritual of Souls:29893).
  if HTConfig:GetIsEnabledBuffAnnouncerHealthStones(nil) == true  then
    if subevent == L["bfm-log-subevent-spell-cast-success"]  then
      if tonumber(spellID) == 29893 then
        HTBuffMonitor:DoBuffAnnouncerHealthStonesAnnouncement(playerName)
        return
      end
    end
  end

  -- "Food Tables" - (Ritual of Refreshment:43987).
  if HTConfig:GetIsEnabledBuffAnnouncerFoodTable(nil) == true  then
    if subevent == L["bfm-log-subevent-spell-cast-success"]  then
      if tonumber(spellID) == 43987 then
        HTBuffMonitor:DoBuffAnnouncerFoodTableAnnouncement(playerName)
        return
      end
    end
  end

  -- "Repair Bots" - (Jeeves:67826), (Field Repair Bot 74A:22700),
  --                 (Field Repair Bot 110G:44389), (Scrapbot:54711).
  if HTConfig:GetIsEnabledBuffAnnouncerRepairBot(nil) == true  then
    if subevent == L["bfm-log-subevent-spell-cast-success"]  then
      if tonumber(spellID) == 67826 or tonumber(spellID) == 22700 or tonumber(spellID) == 44389 or tonumber(spellID) == 54711  then
        HTBuffMonitor:DoBuffAnnouncerRepairBotAnnouncement(playerName)
        return
      end
    end
  end

end

-- ============================================================================
-- == DoBuffAnnouncerFoodAnnouncement(playerName)                            ==
-- ==                                                                        ==
-- == This function announces the deployment of a food buff.                 ==
-- ============================================================================

function HTBuffMonitor:DoBuffAnnouncerFoodAnnouncement(playerName)

  PlaySoundFile("Sound\\interface\\PickUp\\PutDownMeat.wav")
  HTConsole:SendChatMessage(L["bfm-buffannouncer-food-title"], "RAID", nil, nil)
  HTConsole:SendChatMessage(string.gsub(L["bfm-buffannouncer-food-msg-01"], "PLAYER_NAME", playerName), "RAID", nil, nil)

end



-- ============================================================================
-- == DoBuffAnnouncerCauldronAnnouncement(playerName)                        ==
-- ==                                                                        ==
-- == This function announces the deployment of a cauldron.                  ==
-- ============================================================================

function HTBuffMonitor:DoBuffAnnouncerCauldronAnnouncement(playerName)

  PlaySoundFile("Sound\\interface\\PickUp\\PutDownMeat.wav")
  HTConsole:SendChatMessage(L["bfm-buffannouncer-cauldron-title"], "RAID", nil, nil)
  HTConsole:SendChatMessage(string.gsub(L["bfm-buffannouncer-cauldron-msg-01"], "PLAYER_NAME", playerName), "RAID", nil, nil)

end

-- ============================================================================
-- == DoBuffAnnouncerHealthStonesAnnouncement(playerName)                    ==
-- ==                                                                        ==
-- == This function announces the deployment of a soulwell.                  ==
-- ============================================================================

function HTBuffMonitor:DoBuffAnnouncerHealthStonesAnnouncement(playerName)

  PlaySoundFile("Sound\\interface\\PickUp\\PutDownMeat.wav")
  HTConsole:SendChatMessage(L["bfm-buffannouncer-healthstones-title"], "RAID", nil, nil)
  HTConsole:SendChatMessage(string.gsub(L["bfm-buffannouncer-healthstones-msg-01"], "PLAYER_NAME", playerName), "RAID", nil, nil)

end

-- ============================================================================
-- == DoBuffAnnouncerFoodTableAnnouncement(playerName)                       ==
-- ==                                                                        ==
-- == This function announces the deployment of a food table.                ==
-- ============================================================================

function HTBuffMonitor:DoBuffAnnouncerFoodTableAnnouncement(playerName)

  PlaySoundFile("Sound\\interface\\PickUp\\PutDownMeat.wav")
  HTConsole:SendChatMessage(L["bfm-buffannouncer-foodtable-title"], "RAID", nil, nil)
  HTConsole:SendChatMessage(string.gsub(L["bfm-buffannouncer-foodtable-msg-01"], "PLAYER_NAME", playerName), "RAID", nil, nil)

end

-- ============================================================================
-- == DoBuffAnnouncerRepairBotAnnouncement(playerName)                       ==
-- ==                                                                        ==
-- == This function announces the deployment of a repair bot.                ==
-- ============================================================================

function HTBuffMonitor:DoBuffAnnouncerRepairBotAnnouncement(playerName)

  PlaySoundFile("Sound\\interface\\PickUp\\PutDownMeat.wav")
  HTConsole:SendChatMessage(L["bfm-buffannouncer-repairbot-title"], "RAID", nil, nil)
  HTConsole:SendChatMessage(string.gsub(L["bfm-buffannouncer-repairbot-msg-01"], "PLAYER_NAME", playerName), "RAID", nil, nil)

end

-- ============================================================================
-- == DoHeartbeat()                                                          ==
-- ==                                                                        ==
-- == This function is intended to provide a place where non-critical tasks  ==
-- == that need to be performed periodically can be called from. Typically   ==
-- == each HT component may have an instance of this method and each will    ==
-- == be called in turn by the corresponding function in HTConsole which is  ==
-- == considered to be the master-heartbeat component.                       ==
-- ============================================================================

function HTBuffMonitor:DoHeartbeat()

  HTBuffMonitor:DoHeartbeatCheck()

end

-- ============================================================================
-- == DoHeartbeatCheck()                                                     ==
-- ==                                                                        ==
-- == This function checks a player's own buffs and reports the findings via ==
-- == the chat window and the error-frame. It is called via the periodic     ==
-- == heartbeat that occurs (normally) every 5-15 minutes depending upon     ==
-- == configuration.                                                         ==
-- ============================================================================

function HTBuffMonitor:DoHeartbeatCheck()

  if HTConfig:GetIsEnabledBuffMonitor(nil) and HTConfig:GetIsEnabledBuffMonitorCheckOnHeartbeat(nil) then
    HTConsole:SendChatMessage(L["bfm-heartbeatcheck-msg-01"], "CHAT", nil, nil)
    HTBuffMonitor:DoSelfCheck()
    HTConsole:SendChatMessage(L["bfm-heartbeatcheck-msg-02"], "CHAT", nil, nil)
  end

end

-- ============================================================================
-- == DoManualSelfCheck()                                                    ==
-- ==                                                                        ==
-- == This function checks a player's own buffs and reports the findings via ==
-- == the chat window and the error-frame.                                   ==
-- ============================================================================

function HTBuffMonitor:DoManualSelfCheck()

  HTConsole:SendChatMessage(L["bfm-manualselfcheck-msg-01"], "CHAT", nil, nil)
  HTBuffMonitor:DoSelfCheck()
  HTConsole:SendChatMessage(L["bfm-manualselfcheck-msg-02"], "CHAT", nil, nil)

end

-- ============================================================================
-- == DoSelfCheck()                                                          ==
-- ==                                                                        ==
-- == This function checks a player's own buffs and reports the findings via ==
-- == the chat window.                                                       ==
-- ============================================================================

function HTBuffMonitor:DoSelfCheck()

  local characterName = UnitName("player")

  HTBuffMonitor:DoMissingBuffCheck(characterName, "CHAT")
  HTBuffMonitor:DoMissingBuffCheck(characterName, "ERRORFRAME")

end

-- ============================================================================
-- == ListRaidersWithMissingConsumables(channelName)                         ==
-- ==                                                                        ==
-- == This function will output the names of raiders that have missing       ==
-- == consumables.                                                           ==
-- ============================================================================

function HTBuffMonitor:DoRaidCheck(channelName)

  -- Return immediately if the player is not in a raid.
  if not HTCommon:IsPlayerInRaid() then return end

  -- Setup the local variables for text strings and data values.
  local raidCheckBeginningText = L["bfm-raidcheck-msg-01"]
  local raidCheckEndingText    = L["bfm-raidcheck-msg-02"]
  local charIsDeadText         = L["bfm-raidcheck-char-is-dead"]
  local charIsOfflineText      = L["bfm-raidcheck-char-is-offline"]

  local numRaidMembers         = HTData:SafeNumber(GetNumRaidMembers())

  -- Process the raid.
  HTConsole:SendChatMessage(raidCheckBeginningText, channelName)

  for index=1, numRaidMembers, 1 do
    local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(index)
    local isCharacterCheckRequired = true

    -- Is the character dead?
    if isDead == 1 then
      if HTConfig:GetIsEnabledBuffMonitorReportDead(nil) then
        HTConsole:SendChatMessage(string.gsub(charIsDeadText, "CHARACTER_NAME", name), channelName, nil, nil)
      end
      isCharacterCheckRequired = false
    end

    -- Is the character offline?
    if online == nil then
      if HTConfig:GetIsEnabledBuffMonitorReportOffline(nil) then
        HTConsole:SendChatMessage(string.gsub(charIsOfflineText, "CHARACTER_NAME", name), channelName, nil, nil)
      end
      isCharacterCheckRequired = false
    end

    -- Check the character, if we still should.
    if isCharacterCheckRequired then
      HTBuffMonitor:DoMissingBuffCheck(name, channelName)
    end

  end

  HTConsole:SendChatMessage(raidCheckEndingText, channelName)

end
