-- ============================================================================
-- == HTCommon - A collection of common functions used throughout the        ==
-- ==            HalonaTools addon.                                          ==
-- ==                                                                        ==
-- ==            This includes message text display... TBC.                  ==
-- ============================================================================

HTCommon           = LibStub("AceAddon-3.0"):NewAddon("HTCommon", "AceConsole-3.0", "AceEvent-3.0")
local L            = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

local officerRanks = {}           -- Indicates which ranks are 'officer' ranks.

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTCommon:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTCommon:OnEnable()

  HTCommon:ConfigureOfficerRanks()

end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTCommon:OnDisable()
end

-- ============================================================================
-- == GetGuildName()                                                         ==
-- ==                                                                        ==
-- == This function retrieves the Guild name of the user, if they are in a   ==
-- == guild, otherwise it returns "Unguilded".                               ==
-- ============================================================================

function HTCommon:GetGuildName()

  if IsInGuild() then
    local guildName, guildRankName, guildRankIndex = GetGuildInfo("player")
    return guildName
  else
    return L["cmn-guildname-unguilded"]
  end

end

-- ============================================================================
-- == GetGuildRankNameByIndex(index)                                         ==
-- ==                                                                        ==
-- == This function retrieves the name of the guild rank with the supplied   ==
-- == index value. The value supplied is pre-checked and if it is outside    ==
-- == the permitted range, the value "<unused>" is returned instead.         ==
-- ============================================================================

function HTCommon:GetGuildRankNameByIndex(index)

  local numGuildRanks = GuildControlGetNumRanks()

  if index < 1 or index > numGuildRanks then
    return L["cmn-guild-rank-unused"]
  end

  return GuildControlGetRankName(index)

end

-- ============================================================================
-- == GetGuildRankName()                                                     ==
-- ==                                                                        ==
-- == This function retrieves the rank name of the user, if they are in a    ==
-- == guild, otherwise it returns "Unranked".                                ==
-- ============================================================================

function HTCommon:GetGuildRankName()

  if IsInGuild() then
    local guildName, guildRankName, guildRankIndex = GetGuildInfo("player")
    return guildRankName
  else
    return L["cmn-rankname-unguilded"]
  end

end

-- ============================================================================
-- ==  GetGuildLevel()                                                       ==
-- ==                                                                        ==
-- == This function retrieves the level of the player's guild.               ==
-- ============================================================================

function HTCommon:GetGuildLevel()

  if IsInGuild() then
    return GetGuildLevel()
  else
    return L["cmn-rankname-unguilded"]
  end

end

-- ============================================================================
-- == GetGuildOfficerList()                                                  ==
-- ==                                                                        ==
-- == This function retrieves a list of all main-rank "officer" rated        ==
-- == characters in the guild.                                               ==
-- ============================================================================

function HTCommon:GetGuildOfficerList()

  local guildOfficerList = {}

  -- HTConsole:Debug("HTCommon:GetGuildOfficerList() has been called.")

  for index = 1, (GetNumGuildMembers(true)) do
    local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);

    -- HTConsole:Debug("Evaluating character: [" .. charName .. "] with rank: [" .. charRank .. ", rankIndex: [" .. charRankIndex .. "]")

    local rankIndex = charRankIndex + 1

    -- HTConsole:Debug("officerRanks[" .. rankIndex .. "] = [" .. officerRanks[rankIndex] .. "]")
    -- HTConsole:Debug("SafeNumber = [" .. HTData:SafeNumber(officerRanks[rankIndex])  .. "]")

    if HTData:SafeNumber(officerRanks[rankIndex]) == "1" and HTCommon:IsRankIndexTaggedAs(rankIndex, "main") then
      table.insert(guildOfficerList, charName)
      -- HTConsole:Debug("- this character IS an officer and a main!")
    end

  end

  return guildOfficerList

end

-- ============================================================================
-- == FindPlayerByCharacterName(mainCharacterName)                           ==
-- ==                                                                        ==
-- == This function takes a character name as input and searchs through the  ==
-- == guild's roster, looking to find if the player that owns that character ==
-- == is currently online, either on the character named or on an alt        ==
-- == character associated with the character named.                         ==
-- ==                                                                        ==
-- == This assumes that for a given main character, all associated alt       ==
-- == characters will have the main character's name in the public note      ==
-- == field.                                                                 ==
-- ============================================================================

function HTCommon:FindPlayerByCharacterName(characterName)

  characterNameToReturn = characterName .. L["con-findbycharname-offline"]

  totalNumberOfGuildMembers = GetNumGuildMembers(true)

  for index=1, totalNumberOfGuildMembers, 1 do
    name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if name == characterName and online == 1 then
      characterNameToReturn = characterName
    elseif note == characterName and online == 1 then
      characterNameToReturn = name .. L["con-findbycharname-aka"] .. characterName .. ")"
    end
  end

  return characterNameToReturn

end

-- ============================================================================
-- == GetSimpleCharacterName(complexCharacterName)                           ==
-- ==                                                                        ==
-- == This function takes a string, typically assumed to be the output of    ==
-- == FindPlayerByCharacterName(mainCharacterName) function and attempts to  ==
-- == simplify it back from the complex form it may hold to the more simple  ==
-- == form.                                                                  ==
-- ==                                                                        ==
-- == Examples of complexCharacterName conversions.                          ==
-- ==                                                                        ==
-- == Broliant (Offline)         -> Broliant                                 ==
-- == Balgardner (aka: Broliant) -> Balgardner                               ==
-- == Broliant                   -> Broliant                                 ==
-- ==                                                                        ==
-- ============================================================================

function HTCommon:GetSimpleCharacterName(complexCharacterName)

    if HTConfig:GetIsEnabledDebugMode(nil) then
       -- HTConsole:Debug("HTCommon:GetSimpleCharacterName("..complexCharacterName..")")
    end

    if not string.match(complexCharacterName, " ") == nil then
       local Args = HTData:StringSplit(complexCharacterName, " ")
       -- HTConsole:Debug("Returning: "..Args[1]..".")
       return Args[1]
    else
       -- HTConsole:Debug("Returning: "..complexCharacterName..".")
       return complexCharacterName
    end

end

-- ============================================================================
-- == IsRankIndexTaggedAs(rankIndex, tag)                                    ==
-- ==                                                                        ==
-- == This function determines whether the supplied rankIndex value          ==
-- == corresponds to the supplied tag value and returns true if it does and  ==
-- == false otherwise.                                                       ==
-- ==                                                                        ==
-- == Examples:                                                              ==
-- == Is a rankIndex a main rank? : IsRankIndexTaggedAs(rankIndex, "main")   ==
-- == Is a rankIndex a trial rank?: IsRankIndexTaggedAs(rankIndex, "trial")  ==
-- ============================================================================

function HTCommon:IsRankIndexTaggedAs(rankIndex, tag)

  -- HTConsole:Debug("HTCommon:IsRankIndexTaggedAs("..rankIndex..", "..tag..")")

  -- Return immediately if the player is not guilded.
  if L["cmn-guildname-unguilded"] == HTCommon:GetGuildName() then return false end

  -- Return immediately if the supplied rankIndex value is out of range.
  if rankIndex < 1 or rankIndex > 10 then return false end

  local allRankTags = {
    HTConfig:GetRankTags01(nil), HTConfig:GetRankTags02(nil), HTConfig:GetRankTags03(nil),
    HTConfig:GetRankTags04(nil), HTConfig:GetRankTags05(nil), HTConfig:GetRankTags06(nil),
    HTConfig:GetRankTags07(nil), HTConfig:GetRankTags08(nil), HTConfig:GetRankTags09(nil),
    HTConfig:GetRankTags10(nil)
  }

  local rankTags = allRankTags[rankIndex]

  -- HTConsole:Debug("rankIndex=["..rankIndex.."], rankTags=["..rankTags.."]")

  if HTCommon:IsTaggedWith(rankTags, tag) then
    return true
  else
    return false
  end

end

-- ============================================================================
-- == IsTaggedWith(text, tag)                                                ==
-- ==                                                                        ==
-- == This function determines whether the supplied text contains the named  ==
-- == tag.                                                                   ==
-- ============================================================================

function HTCommon:IsTaggedWith(text, tag)

  -- HTConsole:Debug("HTCommon:IsTaggedWith("..text..", "..tag..")")

  local tagList = HTData:StringSplit(text, ",")
  for i,v in ipairs(tagList) do

    if v == tag then

      -- HTConsole:Debug("*** MATCHED TAG "..tag..", returning true")
      return true
    end

  end

  return false

end

-- ============================================================================
-- ==  ConfigureOfficerRanks()                                               ==
-- ==                                                                        ==
-- == This function configures the 'officerRanks' data table to indicate     ==
-- == which of the ten possible guild ranks currently in use are considered  ==
-- == to be 'Officer Ranks'. This is done by assuming that an 'officer' rank ==
-- == is one which has been granted the privilege of allowing the user the   ==
-- == ability to 'speak' in the officer chat channel.                        ==
-- ==                                                                        ==
-- == The resulting 'officerRanks' table will be a 1 dimensional array       ==
-- == containing 10 indexes where officerRanks[1] corresponds to guild rank  ==
-- == 1 (the 'GM' rank) and officerRanks[10] corresponds to guild rank 10.   ==
-- == If a guild rank is an 'officer' rank then the index holds the value of ==
-- == '1', otherwise the index holds the value of '0' indicating the rank is ==
-- == either not an 'officer' rank or not in use.                            ==
-- ==                                                                        ==
-- == It is expected that this function is called when the addon is loaded   ==
-- == or initialised.                                                        ==
-- ============================================================================

function HTCommon:ConfigureOfficerRanks()

  for i=1,10 do
    GuildControlSetRank(i)
    local guildchat_listen, guildchat_speak, officerchat_listen, officerchat_speak, promote, demote, invite_member, remove_member, set_motd, edit_public_note, view_officer_note, edit_officer_note, modify_guild_info, withdraw_repair, withdraw_gold, create_guild_event = GuildControlGetRankFlags()
    table.insert(officerRanks, HTData:SafeNumericBoolean(officerchat_speak))
  end

end

-- ============================================================================
-- ==  IsGuildMasterRank(rankIndex)                                          ==
-- ==                                                                        ==
-- == This function determines whether the supplied rankIndex parameter      ==
-- == represents an "officer-class" guild rank.                              ==
-- ============================================================================

function HTCommon:IsGuildMasterRank(rankIndex)

  if HTData:SafeNumber(rankIndex) == 1 then
    return true
  else
    return false
  end

end

-- ============================================================================
-- ==  IsOfficerRank(rankIndex)                                              ==
-- ==                                                                        ==
-- == This function determines whether the supplied rankIndex parameter      ==
-- == represents an "officer-class" guild rank.                              ==
-- ============================================================================

function HTCommon:IsOfficerRank(rankIndex)

  rankIndex = HTData:SafeNumber(rankIndex)

  local isOfficerRank = false

  if rankIndex < 1 or rankIndex > 10 then
    return isOfficerRank
  end

  GuildControlSetRank(rankIndex)
  local guildchat_listen, guildchat_speak, officerchat_listen, officerchat_speak, promote, demote, invite_member, remove_member, set_motd, edit_public_note, view_officer_note, edit_officer_note, modify_guild_info, withdraw_repair, withdraw_gold, create_guild_event = GuildControlGetRankFlags()

  if HTData:SafeNumericBoolean(officerchat_speak) == 1 then
    isOfficerRank = true
  end

  return isOfficerRank

end

-- ============================================================================
-- ==  IsOfficer(characterName)                                              ==
-- ==                                                                        ==
-- == This function determines whether the character identified by the       ==
-- == supplied name parameter holds an officer level rank within the guild.  ==
-- ============================================================================

function HTCommon:IsOfficer(characterName)

  characterName = HTData:SafeString(characterName)

  for index = 1, (GetNumGuildMembers(true)) do
    local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);

    if charName == characterName then
      if HTData:SafeNumber(officerRanks[charRankIndex+1]) == "1" then
        return true
      end
    end
  end

  return false

end

-- ============================================================================
-- ==  IsCharacterRankedAs(characterName, rankType)                          ==
-- ==                                                                        ==
-- == This function determines whether the rank of the character identified  ==
-- == by the supplied name parameter is accurately described by the supplied ==
-- == rankType parameter. (eg: "main", "alt", "trial" etc) It depends on the ==
-- == existence of accurate rank-tag information for the player's guild.     ==
-- ============================================================================

function HTCommon:IsCharacterRankedAs(characterName, rankType)

  -- HTConsole:Debug("HTCommon:IsCharacterRankedAs("..characterName..", "..rankType..")")

  characterName = HTData:SafeString(characterName)

  for index = 1, (GetNumGuildMembers(true)) do
    local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
    if charName == characterName then
      -- HTConsole:Debug("charName=["..charName.."], charRankIndex=["..charRankIndex.."]")
      if HTCommon:IsRankIndexTaggedAs((tonumber(charRankIndex)+1), rankType) then
        return true
      end
    end
  end

  return false

end


-- ============================================================================
-- ==  IsMainRankedCharacter(characterName)                                  ==
-- ==                                                                        ==
-- == This function determines whether the character identified by the       ==
-- == supplied name parameter is a main ranked character. It depends on the  ==
-- == existence of accurate rank-tag information for the player's guild.     ==
-- ============================================================================

function HTCommon:IsMainRankedCharacter(characterName)

  -- HTConsole:Debug("HTCommon:IsMainRankedCharacter("..characterName..")")
  return HTCommon:IsCharacterRankedAs(characterName, "main")

end

-- ============================================================================
-- ==  IsAltRankedCharacter(characterName)                                   ==
-- ==                                                                        ==
-- == This function determines whether the character identified by the       ==
-- == supplied name parameter is an alt ranked character. It depends on the  ==
-- == existence of accurate rank-tag information for the player's guild.     ==
-- ============================================================================

function HTCommon:IsAltRankedCharacter(characterName)

  return HTCommon:IsCharacterRankedAs(characterName, "alt")

end

-- ============================================================================
-- ==  IsGMRankedCharacter(characterName)                                    ==
-- ==                                                                        ==
-- == This function determines whether the character identified by the       ==
-- == supplied name parameter is a GM ranked character. It depends on        ==
-- == the existence of accurate rank-tag information for the player's guild. ==
-- ============================================================================

function HTCommon:IsGMRankedCharacter(characterName)

  return HTCommon:IsCharacterRankedAs(characterName, "gm")

end

-- ============================================================================
-- ==  IsOfficerRankedCharacter(characterName)                               ==
-- ==                                                                        ==
-- == This function determines whether the character identified by the       ==
-- == supplied name parameter is an officer ranked character. It depends on  ==
-- == the existence of accurate rank-tag information for the player's guild. ==
-- ============================================================================

function HTCommon:IsOfficerRankedCharacter(characterName)

  return HTCommon:IsCharacterRankedAs(characterName, "officer")

end

-- ============================================================================
-- ==  IsMemberRankedCharacter(characterName)                                ==
-- ==                                                                        ==
-- == This function determines whether the character identified by the       ==
-- == supplied name parameter is a member ranked character. It depends on    ==
-- == the existence of accurate rank-tag information for the player's guild. ==
-- ============================================================================

function HTCommon:IsMemberRankedCharacter(characterName)

  return HTCommon:IsCharacterRankedAs(characterName, "member")

end

-- ============================================================================
-- ==  IsTrialRankedCharacter(characterName)                                 ==
-- ==                                                                        ==
-- == This function determines whether the character identified by the       ==
-- == supplied name parameter is a trial ranked character. It depends on the ==
-- == existence of accurate rank-tag information for the player's guild.     ==
-- ============================================================================

function HTCommon:IsTrialRankedCharacter(characterName)

  return HTCommon:IsCharacterRankedAs(characterName, "trial")

end

-- ============================================================================
-- ==  IsRetiredRankedCharacter(characterName)                               ==
-- ==                                                                        ==
-- == This function determines whether the character identified by the       ==
-- == supplied name parameter is a retired ranked character. It depends on   ==
-- == the existence of accurate rank-tag information for the player's guild. ==
-- ============================================================================

function HTCommon:IsRetiredRankedCharacter(characterName)

  return HTCommon:IsCharacterRankedAs(characterName, "retired")

end

-- ============================================================================
-- ==  IsMe(characterName)                                                   ==
-- ==                                                                        ==
-- == This function determines whether the character identified by the       ==
-- == supplied name parameter is the user of the addon.                      ==
-- ============================================================================

function HTCommon:IsMe(characterName)

  local nameToCheck = HTData:SafeString(characterName)
  local myName      = HTData:SafeString(UnitName("player"))

  if nameToCheck == myName then
    return true
  else
    return false
  end

end

-- ============================================================================
-- == IsGuildMemberOnline(characterName)                                     ==
-- ==                                                                        ==
-- == This function performs a simple check to determine whether the named   ==
-- == guild character is currently online.                                   ==
-- ============================================================================

function HTCommon:IsGuildMemberOnline(characterName)

  totalNumberOfGuildMembers = GetNumGuildMembers(true)

  for index=1, totalNumberOfGuildMembers, 1 do
    name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if name == characterName then

      if online == 1 then
        return true
      else
        return false
      end
    end
  end

end

-- ============================================================================
-- == IsGuildMemberAFK(characterName)                                        ==
-- ==                                                                        ==
-- == This function performs a simple check to determine whether the named   ==
-- == guild character is currently AFK.                                      ==
-- ============================================================================

function HTCommon:IsGuildMemberAFK(characterName)

  totalNumberOfGuildMembers = GetNumGuildMembers(true)

  for index=1, totalNumberOfGuildMembers, 1 do
    name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if name == characterName then
      if string.match(string.lower(status), L["cmn-character-status-afk"]) then
        return true
      else
        return false
      end
    end
  end

end

-- ============================================================================
-- == IsGuildMemberDND(characterName)                                        ==
-- ==                                                                        ==
-- == This function performs a simple check to determine whether the named   ==
-- == guild character is currently DND.                                      ==
-- ============================================================================

function HTCommon:IsGuildMemberDND(characterName)

  totalNumberOfGuildMembers = GetNumGuildMembers(true)

  for index=1, totalNumberOfGuildMembers, 1 do
    name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if name == characterName then
      HTConsole:Debug("IsGuildMemberDND(" .. HTData:SafeString(characterName) .. "), status=[" .. HTData:SafeString(status) .. "]")
      if string.match(string.lower(status), L["cmn-character-status-dnd"]) then
        HTConsole:Debug("-> Returning true.")
        return true
      else
        HTConsole:Debug("-> Returning false.")
        return false
      end
    end
  end

end

-- ============================================================================
-- ==  IsCharacterInGuild(characterName)                                     ==
-- ==                                                                        ==
-- == This function determines whether the specified character is in the     ==
-- == player's guild.                                                        ==
-- ============================================================================

function HTCommon:IsCharacterInGuild(characterName)

  -- If the player is not in a guild, no other character can possibly be in the same guild!
  if HTCommon:GetGuildName() == L["cmn-guildname-unguilded"] then return false end

  -- Check if characterName appears in the guild's character roster.
  for index = 1, (GetNumGuildMembers(true)) do

    local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);

    if charName == characterName then
      return true
    end
  end

  return false

end

-- ============================================================================
-- == IsCharacterInParty(characterName)                                      ==
-- ==                                                                        ==
-- == This function determines whether the named character is in the same    ==
-- == party as the person running HT.                                        ==
-- ============================================================================

function HTCommon:IsCharacterInParty(characterName)

  if UnitInParty(characterName) then
     return true
  end

  return false
end

-- ============================================================================
-- == IsCharacterInRaid(characterName)                                       ==
-- ==                                                                        ==
-- == This function determines whether the named character is in the same    ==
-- == raid as the person running HT.                                         ==
-- ============================================================================

function HTCommon:IsCharacterInRaid(characterName)

  if UnitInRaid(characterName) then
     return true
  end

  return false
end

-- ============================================================================
-- == IsCharacterAFriend(characterName)                                      ==
-- ==                                                                        ==
-- == This function determines whether the named character is a friend of    ==
-- == the person running HT.                                                 ==
-- ============================================================================

function HTCommon:IsCharacterAFriend(characterName)

  local numFriends = GetNumFriends()
  local name, level, class, zone, online, status, note

  for i = 1, numFriends, 1 do
    name, level, class, zone, online, status, note = GetFriendInfo(i)
    if online then
      if name == characterName then
        return true
      end
    end
  end

  return false

end

-- ============================================================================
-- == IsCharacterARealIDFriend(characterName)                                ==
-- ==                                                                        ==
-- == This function determines whether the named character is a friend of    ==
-- == the person running HT.                                                 ==
-- ============================================================================

function HTCommon:IsCharacterARealIDFriend(characterName)

  local numFriendsTotal, numFriendsOnline = BNGetNumFriends()
  local presenceID, givenName, surname, toonName, toonID, client, isOnline, lastOnline, isAFK, isDND, broadcastText, noteText, isFriend, broadcastTime

  for i = 1, numFriendsTotal, 1 do
    presenceID, givenName, surname, toonName, toonID, client, isOnline, lastOnline, isAFK, isDND, broadcastText, noteText, isFriend, broadcastTime = BNGetFriendInfo(i)
    if isOnline then
      if toonName == characterName then
        return true
      end
    end
  end

  return false

end

-- ============================================================================
-- ==  IsCharacterRaidLeader(characterName)                                  ==
-- ==                                                                        ==
-- == This function determines whether the named character is the leader of  ==
-- == the raid the player is currently in and returns true if the character  ==
-- == is the raid leader or false otherwise.                                 ==
-- ============================================================================

function HTCommon:IsCharacterRaidLeader(characterName)

  -- Return immediately with false response if player is not in a raid group.
  if not HTCommon:IsPlayerInRaid() then return false end

  for i = 1, GetNumRaidMembers() do
    local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)

    local raidMember = HTData:SafeString(name)
    local raidRank   = HTData:SafeNumber(rank)

    if tostring(string.lower(characterName)) == tostring(string.lower(raidMember)) then
      if tonumber(raidRank) == tonumber("2") then
        return true
      else
        return false
      end
    end
  end

  -- Character not found in raid, therefore return false.
  return false

end

-- ============================================================================
-- ==  IsCharacterRaidAssistant(characterName)                               ==
-- ==                                                                        ==
-- == This function determines whether the named character is an assistant   ==
-- == of the raid the player is currently in and returns true if the         ==
-- == character is an assistant or false otherwise.                          ==
-- ============================================================================

function HTCommon:IsCharacterRaidAssistant(characterName)

  -- Return immediately with false response if player is not in a raid group.
  if not HTCommon:IsPlayerInRaid() then return false end

  for i = 1, GetNumRaidMembers() do
    local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)

    local raidMember = HTData:SafeString(name)
    local raidRank   = HTData:SafeNumber(rank)

    if tostring(string.lower(characterName)) == tostring(string.lower(raidMember)) then
      if tonumber(raidRank) == tonumber("1") then
        return true
      else
        return false
      end
    end
  end

  -- Character not found in raid, therefore return false.
  return false

end

-- ============================================================================
-- ==  IsCharacterRaidMember(characterName)                                  ==
-- ==                                                                        ==
-- == This function determines whether the named character is a member of    ==
-- == the raid the player is currently in and returns true if the character  ==
-- == is a member or false otherwise.                                        ==
-- ============================================================================

function HTCommon:IsCharacterRaidMember(characterName)

  -- Return immediately with false response if player is not in a raid group.
  if not HTCommon:IsPlayerInRaid() then return false end

  for i = 1, GetNumRaidMembers() do
    local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)

    local raidMember = HTData:SafeString(name)
    local raidRank   = HTData:SafeNumber(rank)

    if tostring(string.lower(characterName)) == tostring(string.lower(raidMember)) then
      if tonumber(raidRank) == tonumber("0") then
        return true
      else
        return false
      end
    end
  end

  -- Character not found in raid, therefore return false.
  return false

end

-- ============================================================================
-- ==  IsPlayerInParty()                                                     ==
-- ==                                                                        ==
-- == This function determines whether the player is currently in a party or ==
-- == not and returns true if the player is in a party, false otherwise.     ==
-- ============================================================================

function HTCommon:IsPlayerInParty()

  local numPartyMembers = HTData:SafeNumber(GetNumPartyMembers())

  if numPartyMembers > 0 then
    return true
  else
    return false
  end

end

-- ============================================================================
-- ==  IsPlayerInRaid()                                                      ==
-- ==                                                                        ==
-- == This function determines whether the player is currently in a raid or  ==
-- == not and returns true if the player is in a raid, false otherwise.      ==
-- ============================================================================

function HTCommon:IsPlayerInRaid()

  local numRaidMembers  = HTData:SafeNumber(GetNumRaidMembers())

  if numRaidMembers > 0 then
    return true
  else
    return false
  end

end

-- ============================================================================
-- ==  IsClassInRaid(className)                                              ==
-- ==                                                                        ==
-- == This function determines whether there is a player of the named class  ==
-- == within the raid. Returns true if there is or false otherwise.          ==
-- ============================================================================

function HTCommon:IsClassInRaid(className)

  -- Return immediately with false response if player is not in a raid group.
  if not HTCommon:IsPlayerInRaid() then return false end

  local isClassInRaid = false

  for i = 1, GetNumRaidMembers() do
    local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)

    local raidMember = HTData:SafeString(name)
    local raidClass  = HTData:SafeNumber(class)

    if raidClass == className then
      isClassInRaid = true
    end
  end

  -- Return the indicator value.
  return isClassInRaid

end

-- ============================================================================
-- == ColourByClass(className)                                               ==
-- ==                                                                        ==
-- == This function returns the colour-string suggested by the supplied      ==
-- == className parameter or returs the colour-string representing white.    ==
-- ============================================================================

function HTCommon:ColourByClass(className)

  local colourText = "|cffffffff"

  if className == nil then return colourText end

  if string.lower(className) == string.lower(L["cmn-classname-functional-deathknight"]) then colourText = "|cffc41f3b" end
  if string.lower(className) == string.lower(L["cmn-classname-functional-druid"])       then colourText = "|cffff7d0a" end
  if string.lower(className) == string.lower(L["cmn-classname-functional-hunter"])      then colourText = "|cffabd473" end
  if string.lower(className) == string.lower(L["cmn-classname-functional-mage"])        then colourText = "|cff69ccf0" end
  if string.lower(className) == string.lower(L["cmn-classname-functional-paladin"])     then colourText = "|cfff58cba" end
  if string.lower(className) == string.lower(L["cmn-classname-functional-priest"])      then colourText = "|cffffffff" end
  if string.lower(className) == string.lower(L["cmn-classname-functional-rogue"])       then colourText = "|cfffff569" end
  if string.lower(className) == string.lower(L["cmn-classname-functional-shaman"])      then colourText = "|cff0070de" end
  if string.lower(className) == string.lower(L["cmn-classname-functional-warrior"])     then colourText = "|cffc79c6e" end
  if string.lower(className) == string.lower(L["cmn-classname-functional-warlock"])     then colourText = "|cff9482c9" end

  return colourText

end

-- ============================================================================
-- == GetNumGuildCharacters()                                                ==
-- ==                                                                        ==
-- == This function returns the total number of characters within the guild. ==
-- ============================================================================

function HTCommon:GetNumGuildCharacters()

  return GetNumGuildMembers(true)

end

-- ============================================================================
-- == GetNumGuildCharactersOnline                                            ==
-- ==                                                                        ==
-- == This function returns the number of guild characters that are          ==
-- == currently online.                                                      ==
-- ============================================================================

function HTCommon:GetNumGuildCharactersOnline()

  totalNumberOfGuildMembers = GetNumGuildMembers(true)
  numCharacters             = 0

  for index=1, totalNumberOfGuildMembers, 1 do
    name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if online == 1 then
      numCharacters = numCharacters + 1
    end
  end

  return numCharacters

end

-- ============================================================================
-- == GetNumGuildMembers                                                     ==
-- ==                                                                        ==
-- == This function returns the number of members currently in the guild. It ==
-- == assumes that members will have only one main-ranked character with all ==
-- == other guild characters being alts.                                     ==
-- ============================================================================

function HTCommon:GetNumGuildMembers()

  totalNumberOfGuildMembers = GetNumGuildMembers(true)
  numCharacters             = 0

  for index=1, totalNumberOfGuildMembers, 1 do
    name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsAltRankedCharacter(name) == false then
      numCharacters = numCharacters + 1
    end
  end

  return numCharacters

end

-- ============================================================================
-- == GetNumGuildOfficers()                                                  ==
-- ==                                                                        ==
-- == This function returns the number of characters within the guild that   ==
-- == hold an officer-class main rank. It assumes that officers will have    ==
-- == only one main-ranked character with all other guild characters being   ==
-- == alt-officers.                                                          ==
-- ============================================================================

function HTCommon:GetNumGuildOfficers()

  totalNumberOfGuildMembers = GetNumGuildMembers(true)
  numCharacters             = 0

  for index=1, totalNumberOfGuildMembers, 1 do
    name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsOfficerRankedCharacter(name) == true and HTCommon:IsAltRankedCharacter(name) == false then
      numCharacters = numCharacters + 1
    end
  end

  return numCharacters

end

-- ============================================================================
-- == GetNumGuildOfficersOnline()                                            ==
-- ==                                                                        ==
-- == This function returns the number of characters within the guild that   ==
-- == hold an officer-class main rank and are currently online. It assumes   ==
-- == that officers will have only one main-ranked character with all other  ==
-- == guild characters being alt-officers.                                   ==
-- ============================================================================

function HTCommon:GetNumGuildOfficersOnline()

  totalNumberOfGuildMembers = GetNumGuildMembers(true)
  numCharacters             = 0

  for index=1, totalNumberOfGuildMembers, 1 do
    name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsOfficerRankedCharacter(name) == true and online == 1 then
      numCharacters = numCharacters + 1
    end
  end

  return numCharacters

end

-- ============================================================================
-- == GetNumGuildRaidingMembers()                                            ==
-- ==                                                                        ==
-- == This function returns the number of members currently in the guild who ==
-- == hold a main, non-retired rank. It assumes that members will have only  ==
-- == one main-ranked character with all other guild characters being alts.  ==
-- ============================================================================

function HTCommon:GetNumGuildRaidingMembers()

  totalNumberOfGuildMembers = GetNumGuildMembers(true)
  numCharacters             = 0

  for index=1, totalNumberOfGuildMembers, 1 do
    name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsAltRankedCharacter(name) == false and HTCommon:IsRetiredRankedCharacter(name) == false then
      numCharacters = numCharacters + 1
    end
  end

  return numCharacters

end

-- ============================================================================
-- == GetNumGuildRaidingMembersOnline()                                      ==
-- ==                                                                        ==
-- == This function returns the number of members currently in the guild who ==
-- == hold a main, non-retired rank and are currently online. It assumes     ==
-- == that members will have only one main-ranked character with all other   ==
-- == guild characters being alts.                                           ==
-- ============================================================================

function HTCommon:GetNumGuildRaidingMembersOnline()

  totalNumberOfGuildMembers = GetNumGuildMembers(true)
  numCharacters             = 0

  for index=1, totalNumberOfGuildMembers, 1 do
    name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if HTCommon:IsAltRankedCharacter(name) == false and HTCommon:IsRetiredRankedCharacter(name) == false and online == 1 then
      numCharacters = numCharacters + 1
    end
  end

  return numCharacters

end

-- ============================================================================
-- == GetCurrentGuildPlayMode()                                              ==
-- ==                                                                        ==
-- == This function examines the GMOTD and uses this to determine which of   ==
-- == the available play-modes the guild is currently in.                    ==
-- ==                                                                        ==
-- == Modes: Offtime (O) when no raids/invites taking place. [DEFAULT]       ==
-- ==        Invites (I) when the raid invitation period is running.         ==
-- ==        Raiding (R) when a live raid is currently running.              ==
-- ============================================================================

function HTCommon:GetCurrentGuildPlayMode()

  -- Set the default guild play mode value.
  local guildPlayMode = L["cmn-guild-playmode-offtime"]

  -- == -----------------------------------------------------------------------
  -- == -- Retrieve the GMOTD, decode into component parts. (ie: Words.)     --
  -- == -----------------------------------------------------------------------
  local gmotdActualText     = HTData:SafeString(GetGuildRosterMOTD())
  local gmotdActualWordList = HTData:StringSplit(gmotdActualText, " ")
  local gmotdRIAWordList    = HTData:StringSplit(L["ht-raid-invites-begin-gmotd-01"], " ")
  local gmotdRQAWordList    = HTData:StringSplit(L["ht-raid-session-begin-gmotd-01"], " ")

  local riaMatchText = gmotdRIAWordList[1] .. " " .. gmotdRIAWordList[2] .. " " .. gmotdRIAWordList[3] .. " " .. gmotdRIAWordList[4] .. " " .. gmotdRIAWordList[5] .. " " .. gmotdRIAWordList[6]
  local riaMatchLen  = string.len(riaMatchText)

  local rqaMatchText = gmotdRQAWordList[1] .. " " .. gmotdRQAWordList[2] .. " " .. gmotdRQAWordList[3] .. " " .. gmotdRQAWordList[4] .. " " .. gmotdRQAWordList[5]
  local rqaMatchLen  = string.len(rqaMatchText)

  -- == -----------------------------------------------------------------------
  -- == -- Test the GMOTD: Is the raid invitation period open?               --
  -- == -----------------------------------------------------------------------
  if string.len(gmotdActualText) > riaMatchLen then
    if string.sub(gmotdActualText, 1, riaMatchLen) == riaMatchText then
      guildPlayMode = L["cmn-guild-playmode-invites"]
    end
  end

  -- == -----------------------------------------------------------------------
  -- == -- Test the GMOTD: Is a raid currently in progress?                  --
  -- == -----------------------------------------------------------------------
  if string.len(gmotdActualText) > rqaMatchLen then
    if string.sub(gmotdActualText, 1, rqaMatchLen) == rqaMatchText then
      guildPlayMode = L["cmn-guild-playmode-raiding"]
    end
  end

  return guildPlayMode

end

-- ============================================================================
-- == GetCurrentGuildPlayModeName()                                          ==
-- ==                                                                        ==
-- == This function calls the GetCurrentGuildPlayMode() function and         ==
-- == converts the response to a corresponding name value which is returned. ==
-- ==                                                                        ==
-- == Modes: Offtime (O) -> "Offtime" [DEFAULT]                              ==
-- ==        Invites (I) -> "Invites"                                        ==
-- ==        Raiding (R) -> "Raiding"                                        ==
-- ============================================================================

function HTCommon:GetCurrentGuildPlayModeName()

  local playMode     = HTCommon:GetCurrentGuildPlayMode()
  local playModeName = L["cmn-guild-playmode-offtime-name"]

  if playMode == L["cmn-guild-playmode-invites"] then

    playModeName = L["cmn-guild-playmode-invites-name"]

  elseif playMode == L["cmn-guild-playmode-raiding"] then

    playModeName = L["cmn-guild-playmode-raiding-name"]

  end

  return playModeName

end

-- ============================================================================
-- == GetMainCharacterFromAltCharacter(characterName)                        ==
-- ==                                                                        ==
-- == This function retrieves the name of the main character with which the  ==
-- == supplied alt character name is associated.                             ==
-- ==                                                                        ==
-- == It is assumed that the main character's name will be in the public     ==
-- == note field of the supplied alt character.                              ==
-- ============================================================================

function HTCommon:GetMainCharacterFromAltCharacter(characterName)

  -- First Check: Is the character an alt-ranked character?
  if HTCommon:IsAltRankedCharacter(characterName) == false then
    return characterName
  end

  -- Find the main character with which this character is associated.
  local totalNumberOfGuildMembers = GetNumGuildMembers(true)
  local characterList             = {}

  for index=1, totalNumberOfGuildMembers do
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(index)
    if name == characterName then
      if HTCommon:IsCharacterInGuild(note) == true and HTCommon:IsMainRankedCharacter(note) == true then
        return note
      else
        return characterName
      end
    end
  end

  -- No matches, so return the supplied characterName as default value.
  return characterName

end

-- ============================================================================
-- == GetMasterLooterCharacterName()                                         ==
-- ==                                                                        ==
-- == This function attempts to locate the name of the current master looter ==
-- == and will return the name if it locates it, otherwise it returns the    ==
-- == standard '<unset>' value.                                              ==
-- ============================================================================

function HTCommon:GetMasterLooterCharacterName()

   local numberOfRaidMembers = GetNumRaidMembers()
   local masterLooterName    = L["all-data-unset"]

   for index=1, numberOfRaidMembers, 1 do
      name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(index)

      if isML then
         masterLooterName = name
         return masterLooterName
      end
   end

   return masterLooterName

end

-- ============================================================================
-- == GetBestMasterLooterCandidateName()                                     ==
-- ==                                                                        ==
-- == This function attempts to determine which member of the current raid   ==
-- == is the best candidate for the position of Master Looter. Various       ==
-- == criteria may be used to make this determination as follows:            ==
-- ==                                                                        ==
-- == -> Is candidate a raid assistant?                                      ==
-- == -> Is candidate a guild officer?                                       ==
-- == -> Is candidate a guild master?                                        ==
-- == -> Is candidate a designated master looter? (ie: "ML" in officer note) ==
-- ============================================================================

function HTCommon:GetBestMasterLooterCandidateName()

  local numberOfRaidMembers = GetNumRaidMembers()
  local masterLooterName    = HTCommon:GetMasterLooterCharacterName()
  local candidateScore      = 0
  local candidateName       = masterLooterName

  for index=1, numberOfRaidMembers, 1 do
    name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(index)

    local characterName  = HTData:SafeString(name)
    local characterScore = 0

    -- Perform the suite of score-boosting tests...
    if HTConfig:GetIsEnabledASMLRaidLeaderTest(nil)             and HTCommon:IsCharacterRaidLeader(characterName)             then characterScore = characterScore + 10 end
    if HTConfig:GetIsEnabledASMLRaidAssistantTest(nil)          and HTCommon:IsCharacterRaidAssistant(characterName)          then characterScore = characterScore + 10 end
    if HTConfig:GetIsEnabledASMLGuildMasterTest(nil)            and HTCommon:IsGMRankedCharacter(characterName)               then characterScore = characterScore + 10 end
    if HTConfig:GetIsEnabledASMLGuildOfficerTest(nil)           and HTCommon:IsOfficerRankedCharacter(characterName)          then characterScore = characterScore + 10 end
    if HTConfig:GetIsEnabledASMLDesignatedMasterLooterTest(nil) and HTCommon:IsCharacterDesignatedMasterLooter(characterName) then characterScore = characterScore + 50 end

    -- Is this character's score higher than the current highest?
    if tonumber(characterScore) > tonumber(candidateScore) then
      candidateScore = characterScore
      candidateName  = characterName
    end

  end

  -- If for some mad reason no suitable candidate was found, return the
  -- master looter name defined initially. This really SHOULD NOT happen.
  if candidateName == L["all-data-unset"] then
    return masterLooterName
  else
    return candidateName
  end

end

-- ============================================================================
-- == IsCharacterDesignatedMasterLooter(characterName)                       ==
-- ==                                                                        ==
-- == This function attempts to determine whether the character identified   ==
-- == by the supplied parameter is a designated master looter. This is done  ==
-- == by retrieving the character's officer note and matching for "ML" in    ==
-- == its content.                                                           ==
-- ============================================================================

function HTCommon:IsCharacterDesignatedMasterLooter(characterName)

  -- Return false immediately if the character is NOT in the guild.
  if HTCommon:IsCharacterInGuild(characterName) == false then return false end

  -- Check if characterName appears in the guild's character roster.
  for index = 1, (GetNumGuildMembers(true)) do

    local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);

    if charName == characterName then
      local officerNoteField = HTData:SafeString(charOfficerNote)
      local officerNoteWords = HTData:StringSplit(officerNoteField, " ")

      for i,v in ipairs(officerNoteWords) do
        if v == L["cmn-designated-master-looter-tag"] then
          return true
        end
      end

      -- We scanned all the words in the officer note, nothing matched.
      return false
    end

  end

  return false

end

-- ============================================================================
-- == DoReadyCheck()                                                         ==
-- ==                                                                        ==
-- == This function attempts to perform a ready check IF the user is in a    ==
-- == raid group and is either the raid-leader or a raid-assistant.          ==
-- ============================================================================

function HTCommon:DoReadyCheck()

  local characterName = UnitName("player")

  if HTCommon:IsPlayerInRaid() then
    if HTCommon:IsCharacterRaidAssistant(characterName) or HTCommon:IsCharacterRaidLeader(characterName) then
      HTConsole:SendChatMessage(L["cmn-ready-check-performing-now"], "CHAT", nil, nil)
      DoReadyCheck()
    else
      HTConsole:SendChatMessage(L["cmn-ready-check-not-promoted"], "CHAT", nil, nil)
    end
  else
    HTConsole:SendChatMessage(L["cmn-ready-check-not-in-raid"], "CHAT", nil, nil)
  end

end

-- ============================================================================
-- ==  DisbandRaid()                                                         ==
-- ==                                                                        ==
-- == This function attempts to disband the current raid group if certain    ==
-- == conditions are met as follows:                                         ==
-- ==                                                                        ==
-- == -> The player is in a raid group.                                      ==
-- == -> The player is the raid leader.                                      ==
-- ============================================================================

function HTCommon:DisbandRaid()

  -- Return immediately with false response if player is not in a raid group.
  if not HTCommon:IsPlayerInRaid() then return false end

  -- If the player is NOT the raid leader, advise.
  if HTCommon:IsCharacterRaidLeader(UnitName("player")) == false then
    HTConsole:SendChatMessage(L["cmn-disband-raid-only-if-raid-leader"], "CHAT", nil, nil)
    return
  end

  -- If the guild's play mode is either 'invites' or 'raiding' then cleanly end
  -- the raid session first, before disbanding the raid.
  if HTCommon:GetCurrentGuildPlayMode() == L["cmn-guild-playmode-invites"] or HTCommon:GetCurrentGuildPlayMode() == L["cmn-guild-playmode-raiding"] then
    HalonaTools:EndRaidSessionQuietly(UnitName("player"))
  end

  -- Iterate through the raid members, removing them one at a time.
  for i = 1, GetNumRaidMembers() do
    local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
    local characterName = HTData:SafeString(name)

    -- Uninvite the member from the raid if the member is not 'me'.
    if HTCommon:IsMe(characterName) == false then
      UninviteUnit(characterName)
    end
  end

  -- Finally, leave the party yourself.
  LeaveParty()

  HTConsole:SendChatMessage(L["cmn-disband-raid-all-done"], "CHAT", nil, nil)

  return

end
